package com.winhc.phoenix.example.controller;

import com.winhc.phoenix.example.util.JsonUtils;
import com.winhc.phoenix.example.vo.SearchIndexModel;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2022/9/14 16:05
 */
@Controller
@RequestMapping("index")
@AllArgsConstructor
public class IndexController {

    private final HttpServletRequest request;


    @RequestMapping("/search-{template}.html")
    public String search(Model model, @PathVariable String template) {
        String serverName = request.getServerName();
        String scheme = request.getScheme();
        int serverPort = request.getServerPort();

        String prefix = scheme + "://" + serverName + ":" + serverPort;

        Map<String, SearchIndexModel> models =
                SearchIndexModel.getModels();
        SearchIndexModel searchIndexModel = models.get(template);

        String columnsJs = searchIndexModel.getColumnsJs().stream().map(SearchIndexModel.ColumnInfo::toString).collect(Collectors.joining(",\n", "[", "]"));
        String highlightMapping = JsonUtils.jsonObjToString(searchIndexModel.getHighlightMapping());

        model.addAttribute("title", searchIndexModel.getTitle());
        model.addAttribute("formHtml", StringUtils.defaultString(searchIndexModel.getFormHtml(), SearchIndexModel.defaultFormHtml));
        model.addAttribute("historyForm", StringUtils.defaultString(searchIndexModel.getFormHtml(), ""));
        model.addAttribute("initJs", StringUtils.defaultString(searchIndexModel.getInitJs()));
        model.addAttribute("method", searchIndexModel.getMethod());
        model.addAttribute("url", prefix + searchIndexModel.getUrl());
        model.addAttribute("queryParamsFunc", StringUtils.defaultString(searchIndexModel.getQueryParamsFunc(), SearchIndexModel.defaultQueryParamsFunc));
        model.addAttribute("responseHandlerFunc", searchIndexModel.getResponseHandlerFunc());
        model.addAttribute("columnsJs", columnsJs);
        model.addAttribute("highlightMapping", highlightMapping);
        model.addAttribute("jsFunction", String.join("\n", searchIndexModel.getJsFunction()));
        return "search-templates";
    }

}
